from flask import Blueprint, request, jsonify, make_response, json, session
from bcrypt import checkpw
from app.customer.models import Customer
from app.Site.models import Site
from app.rooms.models import Room
from app.devices.models import Device
from app.gateways.models import Gateways
from app.roles.models import Role
import bcrypt
from app.utils import add_logo, get_logo_from_logotable
from app import db
from app.users.models import User
from app.users.schemas import user_schema, login_schema, password_schema, user_update_schema
from app.utils import generate_user_jwt_token, token_required, paginated_query, delete_record, single_record_query

users = Blueprint('users', __name__)
password = Blueprint('password', __name__)

@users.route('/api/v1/user/add', methods=['POST'])
@token_required(1, 2)
def add_user(current_user):
    try:
        data_dict = json.loads(request.form.get('formData'))
        if data_dict['device_name'] == "Device not available":
            data_dict['device_name'] = ""
        login_data = user_schema.load({
            'first_name': data_dict['first_name'].strip(),
            'last_name': data_dict['last_name'].strip(),
            'role_id': str(data_dict['role_id']),
            'device_name': data_dict['device_name'],
            'device_id': str(data_dict['device_id']),
            'customer_id': request.args.get('customer_id'),
            'zone_id': current_user.zone_id,
            'room_id': current_user.room_id,
            'site_id': request.args.get('site_id'),
            'preference': '[0,1]',
            'logo_id': add_logo(request.files.get('image')),
            'email': data_dict['email'].strip(),
            'password': data_dict['password'].strip(),
            'confirm_password': data_dict['confirm_password'].strip()
        })
        full_name = json.dumps({"first_name": login_data['first_name'], "last_name": login_data['last_name']})
        login_data.update({"full_name": full_name})
        login_data.pop('confirm_password')
        login_data.pop('first_name')
        login_data.pop('last_name')
        login_data.pop('device_name')
        db.session.add(User(**login_data))
        db.session.commit()
        return make_response(jsonify({'message': 'Staff has been added successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error create user'}), 500)

@users.route('/api/v1/get/users', methods=['GET'])
@token_required(1, 2)
def get_users(current_user):
    try:
        users_query = User.query.order_by(User.full_name.asc())
        users_paginated, data = paginated_query(users_query,
            [User.full_name], User.site_id)
        if users_paginated:
            serialized_users = []
            for user in users_paginated:
                if user.id is 1:
                    continue
                logo = get_logo_from_logotable(user.logo_id) if user.logo_id else ("", "")
                full_name = json.loads(user.full_name.replace("'", '"'))
                device = Device.query.filter_by(device_id=user.device_id).first()
                serialized_users.append({
                    'id': user.id,
                    'role_id': user.role_id,
                    'customer_id': user.customer_id,
                    'full_name': user.full_name,
                    'email': user.email,
                    'device_id': user.device_id,
                    'device_name': device.device_name if device else "",
                    'userImg': logo[0],
                    'img_content_type': logo[1],
                    'first_name': full_name.get('first_name'),
                    'last_name': full_name.get('last_name'),
                    'role_name': Role.query.filter_by(id=user.role_id).first().role_name
                })
            return jsonify({'users': serialized_users, 'data': data}), 200
        else:
            return jsonify({'message': 'No users found', 'data': data}), 200
    except Exception as e:
        return jsonify({'message': f'Error: {e}', 'status': 'error'}), 500

@users.route('/api/v1/get/user/<int:id>', methods=['GET'])
@token_required(1, 2,3)
def get_user(current_user, id):
    try:
        user = single_record_query(User, User.id, id, current_user)
        if not user:
            return make_response(jsonify({'message': 'User not found', "status": "error"}), 404)
        full_name = json.loads(user.full_name.replace("'", '"'))
        device = Device.query.filter_by(device_id=user.device_id).first() if user.device_id else None
        serialized_users = {
            'id': user.id,
            'role_id': user.role_id,
            'customer_id': user.customer_id,
            'device_id': user.device_id,
            'device_name': device.device_name if device else "",
            'full_name': user.full_name,
            'email': user.email,
            'first_name': full_name.get('first_name'),
            'last_name': full_name.get('last_name'),
            'middle_name': full_name.get('middle_name'),
            'role_name': Role.query.filter_by(id=user.role_id).first().role_name if user.role_id else ""
        }
        data = ("", "")
        if user.logo_id:
            data = get_logo_from_logotable(user.logo_id)
        userImg = {
            'logo': data[0],
            'logo_content_type': data[1],
        }
        return make_response(jsonify({'users': serialized_users, "userImg": userImg}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)

@users.route('/api/v1/update/user/<int:id>', methods=['PUT'])
@token_required(1, 2)
def update_user(current_user, id):
    try:
        user = single_record_query(User, User.id, id, current_user)
        data_dict = json.loads(request.form.get('formData'))
        login_data = user_schema.load({
            'first_name': data_dict['first_name'].strip(),
            'last_name': data_dict['last_name'].strip(),
            'device_name': data_dict['device_name'],
            'device_id': str(data_dict['device_id']),
            'role_id': str(data_dict['role_id']),
            'customer_id': data_dict['customer_id'],
            'email': data_dict['email'],
            'preference': '[0,1]',
            'password': data_dict['password'].strip(),
            'confirm_password': data_dict['confirm_password'].strip(),
            'zone_id': current_user.zone_id,
            'room_id': current_user.room_id,
            'site_id': current_user.site_id,
        })
        user.full_name = json.dumps({"first_name": login_data['first_name'],
                                     "last_name": login_data['last_name']})
        user.email = login_data['email']
        user.role_id = login_data['role_id']
        user.device_id = login_data['device_id']
        user.logo_id = add_logo(request.files['image']) if 'image' in request.files else user.logo_id
        if data_dict['password']:
            user.password = bcrypt.hashpw(login_data['password'].encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
        db.session.commit()
        return make_response(jsonify({'message': 'Staff has been updated Successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error update staffs'}), 500)

@users.route('/api/v1/delete/user/<int:id>', methods=['DELETE'])
@token_required(1, 2)
def delete_user(current_user, id):
    return delete_record(User, current_user, id, "Staff", "id")

@users.route('/login', methods=['POST'])
def login():
    try:
        schemaArray = dict()
        data_dict = json.loads(request.get_json()['formData'])
        schemaArray['email'] = data_dict['email']
        schemaArray['password'] = data_dict['password']
        loaded_data = login_schema.load(schemaArray)
        user = User.query.filter_by(email=loaded_data['email']).first()
        if user and checkpw(loaded_data['password'].encode('utf-8'), user.password.encode('utf-8')):
            token = generate_user_jwt_token(user.id)
            return make_response(jsonify(
                {"message": "Login successfully", 'customer_id': user.customer_id, 'site_id': user.site_id,
                 'token': token, "status": "Success"}), 200)
        else:
            return make_response(jsonify({'message': 'Invalid credentials', "status": "error"}), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'error Login user', 'args': e.args, "status": "error"}), 500)

@users.route('/logout', methods=['GET'])
def logout():
    try:
        device_identifier = request.remote_addr
        if device_identifier:
            session.pop(f'token_{device_identifier}', None)
        response = make_response(jsonify({"message": "Logout successful"}), 200)
        response.delete_cookie('session_token')
        db.session.commit()
        return response
    except Exception as e:
        return make_response(jsonify({"message": "An error occurred while logging out"}), 500)

@users.route('/api/v1/profile_information', methods=['GET'])
@token_required(1, 2, 3)
def profile_information(current_user):
    try:
        user = User.query.filter_by(id=current_user.id).first()
        full_name = json.loads(user.full_name.replace("'", "'"))
        data = get_logo_from_logotable(user.logo_id) if user.logo_id else ("", "")
        if user:
            return make_response(jsonify({"user": {
                    "id": user.id,
                    "first_name":  full_name.get('first_name'),
                    "full_name": user.full_name,
                    "email": user.email,
                    "site_id": user.site_id,
                    'userImg': data[0],
                    'img_content_type': data[1],
                    "customer_id": user.customer_id,
                    "site_name": Site.query.filter_by(site_id=current_user.site_id).first().site_name,
                    "zone_id": user.zone_id,
                    "preference": user.preference,
                    "customer_name": Customer.query.filter_by(customer_id=current_user.customer_id).first().customer_name,
                    "role_id": user.role_id
                }}))
    except Exception as e:
        return make_response(jsonify({'args': e.args, 'status': 'error', 'message': 'error profile information'}))

@users.route('/api/v1/get/dashboard', methods=['GET'])
@token_required(1, 2, 3)
def dashboard(current_user):
    try:
        site_id = request.args.get('site_id')
        if not site_id:
            return make_response(jsonify({"status": "error", "message": "site_id is required"}), 400)
        return jsonify({"dashboard": {
            "user_count": User.query.filter_by(site_id=site_id).count(),
            "total_room": Room.query.filter_by(site_id=site_id).count(),
            "total_device": Device.query.filter_by(site_id=site_id).count(),
            "total_gateway": Gateways.query.filter_by(site_id=site_id).count(),
            "site_id": site_id,
            "customer_id": request.args.get('customer_id'),
            "role_id": current_user.role_id
        }})
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error user dashboard'}), 500)

@users.route("/api/v1/update_password/<int:id>", methods=["PUT"])
@token_required(1, 2, 3)
def update_password(user_details, id):
    try:
        user = User.query.filter_by(id=id).first()
        if user:
            data_dict = json.loads(request.form.get('formData'))
            schemaArray = {
                'password': data_dict['password'],
                'confirm_password': data_dict['confirm_password']
            }
            loaded_data = password_schema.load(schemaArray)
            password = bcrypt.hashpw(loaded_data['password'].encode('utf-8'), bcrypt.gensalt()).decode('utf-8')
            db.session.query(User).filter(User.id == id).update({
                User.password: password
            })
            db.session.commit()
            return make_response(
                jsonify({'message': 'Profile password  has been updated Successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error update staffs'}), 500)

@users.route('/api/v1/update_user_profile/<int:id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_user_profile(current_user, id):
    try:
        user = single_record_query(User, User.id, id, current_user)
        data_dict = json.loads(request.form.get('formData'))
        schema_Array = {
            'first_name': data_dict['first_name'].strip(),
            'last_name': data_dict['last_name'].strip(),
            'role_id': data_dict['role_id'],
            'email': data_dict['email'],
        }
        login_data = user_update_schema.load(schema_Array)
        user.full_name = json.dumps({"first_name": login_data['first_name'],
                                     "last_name": login_data['last_name']})
        user.email = login_data['email']
        user.role_id = login_data['role_id']
        user.logo_id = add_logo(request.files['image']) if 'image' in request.files else user.logo_id,
        db.session.commit()
        data = get_logo_from_logotable(user.logo_id) if user.logo_id else ("", "")
        return make_response(jsonify({
            'message': 'Profile has been updated Successfully',
            "status": "Success",
            "userImg": data[0],
            "img_content_type": data[1]
        }), 200)

    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error update staffs'}), 500)

@users.route("/update-zone/<int:id>", methods=["PUT"])
@token_required(1, 2, 3)
def update_zone_user(current_user, id):
    try:
        User.query.filter(User.id == id).update({
            User.zone_id: json.loads(request.form.get('formData'))['zone_id']})
        db.session.commit()
        return make_response(jsonify({'message': 'Active zone updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error data'}), 500)

@users.route("/update-preference/<int:id>", methods=["PUT"])
@token_required(1, 2, 3)
def update_prefernce_user(current_user, id):
    try:
        User.query.filter(User.id == id).update({
            User.preference: json.loads(request.form.get('formData'))['preference_flag']})
        db.session.commit()
        return make_response(jsonify({'message': 'Active preference updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "status": "error", "message": 'error data'}), 500)